"""
Configuration of Doconce parameters that can also be set
on the command line.
For example, a line in this file:

some_option = some_value

corresponds to a command-line option --some_option=some_value
"""

help = False
debug = False
no_abort = False
#verbose =
#language =
preprocess_include_subst = False
#syntax_check =
skip_inline_comments = False
draft = False
#CC_license =
align2equations = False
force_tikz_conversion = False
#tikz_libs =
IBPLOT = False
#exercise_numbering =
exercises_in_zip = False
#exercises_in_zip_filename =
#toc_depth =
#encoding =
no_ampersand_quote = False
no_mako = False
no_preprocess = False
mako_strict_undefined = False
no_header_footer = False
no_emoji = False
siunits = False
allow_refs_to_external_docs = False
runestone = False
#max_bc_linelength =
#replace_ref_by_latex_auxno =
keep_pygments_html_bg = False
#minted_latex_style =
#pygments_html_style =
pygments_html_linenos = False
xhtml = False
#html_output =
#html_style =
#html_template =
no_title = False
#html_code_style =
#html_pre_style =
#html_toc_indent =
#html_body_style =
#html_body_font =
#html_heading_font =
#html_video_autoplay =
#html_admon =
html_admon_shadow = False
#html_admon_bg_color =
#html_admon_bd_color =
#css =
html_box_shadow = False
#html_share =
#html_exercise_icon =
#html_exercise_icon_width =
#html_raw_github_url =
html_DOCTYPE = False
html_links_in_new_window = False
#html_quiz_button_text =
#html_bootstrap_navbar =
#html_bootstrap_jumbotron =
#html_bootstrap_navbar_links =
#html_figure_caption =
#html_figure_hrule =
#html_copyright =
html_responsive_figure_width = False
cite_doconce = False
#device =
number_all_equations = False
denumber_all_equations = False
#latex_style =
#latex_font =
#latex_code_style =
#latex_code_leftmargin =
#latex_code_bg =
latex_code_bg_vpad = False
#latex_code_lststyles =
#latex_copyright =
#latex_bibstyle =
#section_numbering =
#latex_table_format =
#latex_table_row_sep =
#latex_title_layout =
#latex_link_color =
#latex_title_reference =
#latex_encoding =
#latex_packages =
#latex_papersize =
#latex_list_of_exercises =
#latex_movie =
#latex_movie_controls =
latex_external_movie_viewer = False
latex_fancy_header = False
#latex_section_headings =
#latex_colored_table_rows =
latex_line_numbers = False
latex_todonotes = False
latex_double_spacing = False
latex_labels_in_margin = False
latex_index_in_margin = False
#latex_preamble =
latex_no_program_footnotelink = False
#latex_admon =
#latex_admon_color =
latex_admon_title_no_period = False
#latex_admon_envir_map =
#latex_subex_header_postfix =
xelatex = False
latex_double_hyphen = False
#latex_elsevier_journal =
#ipynb_version =
#ipynb_split_pyshell =
#ipynb_cite =
#ipynb_admon =
#ipynb_figure =
#ipynb_movie =
verbose = False
examples_as_exercises = False
exercises_as_subsections = False
solutions_at_end = False
without_solutions = False
answers_at_end = False
without_answers = False
without_hints = False
#exercise_solution =
wordpress = False
tables2csv = False
sections_up = False
sections_down = False
#os_prompt =
#code_skip_until =
#code_prefix =
#figure_prefix =
#movie_prefix =
no_mp4_webm_ogg_alternatives = False
handout = False
urlcheck = False
#labelcheck =
#short_title =
markdown = False
#md2do_output =
github_md = False
slate_md = False
strapdown = False
#bootswatch_theme =
strict_markdown_output = False
multimarkdown_output = False
#quiz_question_prefix =
#quiz_choice_prefix =
#quiz_horizontal_rule =
#quiz_explanations =
rst_uio = False
rst_mathjax = False
sphinx_keep_splits = False
#sphinx_figure_captions =
oneline_paragraphs = False
